@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 # <examples/doc_with_itercb.py>
 import matplotlib.pyplot as plt
 from numpy import linspace, random
@@ -8,13 +6,13 @@ from lmfit.lineshapes import gaussian
 from lmfit.models import GaussianModel, LinearModel
 
 
-def per_iteration(pars, iter, resid, *args, **kws):
-    print(" ITER ", iter, ["%.5f" % p for p in pars.values()])
+def per_iteration(pars, iteration, resid, *args, **kws):
+    print(" ITER ", iteration, ["%.5f" % p for p in pars.values()])
 
 
 x = linspace(0., 20, 401)
 y = gaussian(x, amplitude=24.56, center=7.6543, sigma=1.23)
-y = y - .20*x + 3.333 + random.normal(scale=0.23, size=len(x))
+y = y - .20*x + 3.333 + random.normal(scale=0.23, size=x.size)
 
 mod = GaussianModel(prefix='peak_') + LinearModel(prefix='bkg_')
 
@@ -32,6 +30,7 @@ plt.plot(x, y, 'b--')
 print('Nfev = ', out.nfev)
 print(out.fit_report())
 
-plt.plot(x, out.best_fit, 'k-')
+plt.plot(x, out.best_fit, 'k-', label='best fit')
+plt.legend(loc='best')
 plt.show()
 # <end examples/doc_with_itercb.py>
