@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 # <examples/doc_parameters_valuesdict.py>
 import numpy as np
 
@@ -7,8 +5,8 @@ from lmfit import Minimizer, Parameters, report_fit
 
 # create data to be fitted
 x = np.linspace(0, 15, 301)
-data = (5. * np.sin(2*x - 0.1) * np.exp(-x*x*0.025) +
-        np.random.normal(size=len(x), scale=0.2))
+data = (5.0 * np.sin(2.0*x - 0.1) * np.exp(-x*x*0.025) +
+        np.random.normal(size=x.size, scale=0.2))
 
 
 # define objective function: returns the array to be minimized
@@ -27,7 +25,7 @@ params.add('decay', value=0.1)
 params.add('shift', value=0.0, min=-np.pi/2., max=np.pi/2)
 params.add('omega', value=3.0)
 
-# do fit, here with leastsq model
+# do fit, here with the default leastsq algorithm
 minner = Minimizer(fcn2min, params, fcn_args=(x, data))
 result = minner.minimize()
 
