@@ -23,10 +23,10 @@ def test_import_from(obj):
 objects = [('test_string', (str,)),
            (np.array([7.0]), np.ndarray),
            (np.array([1.0+2.0j]), np.ndarray),
-           (123.456, np.float),
-           (10, np.float),
+           (123.456, float),
+           (10, int),
            ('café', (str,)),
-           (10.0-5.0j, np.complex),
+           (10.0-5.0j, complex),
            (['a', 'b', 'c'], list),
            (('a', 'b', 'c'), tuple),
            ({'a': 1.0, 'b': 2.0, 'c': 3.0}, dict),
