@@ -40,16 +40,26 @@ def test_check_ast_errors():
     """Assert that an exception is raised upon AST errors."""
     pars = lmfit.Parameters()
 
-    msg = r"at expr='<_ast.Module object at"
+    msg = r"at expr='<_?ast.Module object at"
     with pytest.raises(NameError, match=msg):
         pars.add('par1', expr='2.0*par2')
 
 
-def test_parameters_init():
-    """Test for initialization of the Parameters class"""
+def test_parameters_init_with_asteval():
+    """Test for initialization of the Parameters class with asteval."""
     ast_int = asteval.Interpreter()
-    pars = lmfit.Parameters(asteval=ast_int, usersyms={'test': np.sin})
+
+    msg = ("The use of the 'asteval' argument for the Parameters class was "
+           "deprecated in lmfit v0.9.12 and will be removed in a later "
+           "release. Please use the 'usersyms' argument instead!")
+    with pytest.warns(FutureWarning, match=msg):
+        pars = lmfit.Parameters(asteval=ast_int)
     assert pars._asteval == ast_int
+
+
+def test_parameters_init_with_usersyms():
+    """Test for initialization of the Parameters class with usersyms."""
+    pars = lmfit.Parameters(asteval=None, usersyms={'test': np.sin})
     assert 'test' in pars._asteval.symtable
 
 
